@@ -1,16 +1,24 @@
 # This module should be kept compatible with Python 2.1.
+"""distutils.command.install_lib
 
-__revision__ = "$Id: install_lib.py 37828 2004-11-10 22:23:15Z loewis $"
+Implements the Distutils 'install_lib' command
+(install all Python modules)."""
 
-import sys, os, string
+__revision__ = "$Id: install_lib.py 81308 2010-05-18 23:37:50Z georg.brandl $"
+
+import os
 from types import IntType
+import sys
+
 from distutils.core import Command
 from distutils.errors import DistutilsOptionError
 
 
 # Extension for Python source files.
-PYTHON_SOURCE_EXTENSION = os.extsep + "py"
-
+if hasattr(os, 'extsep'):
+    PYTHON_SOURCE_EXTENSION = os.extsep + "py"
+else:
+    PYTHON_SOURCE_EXTENSION = ".py"
 
 class install_lib (Command):
 
@@ -78,7 +86,8 @@ class install_lib (Command):
         if type(self.optimize) is not IntType:
             try:
                 self.optimize = int(self.optimize)
-                assert 0 <= self.optimize <= 2
+                if self.optimize not in (0, 1, 2):
+                    raise AssertionError
             except (ValueError, AssertionError):
                 raise DistutilsOptionError, "optimize must be 0, 1, or 2"
 
@@ -119,6 +128,10 @@ class install_lib (Command):
         return outfiles
 
     def byte_compile (self, files):
+        if sys.dont_write_bytecode:
+            self.warn('byte-compiling is disabled, skipping.')
+            return
+
         from distutils.util import byte_compile
 
         # Get the "--root" directory supplied to the "install" command,
